package eu.eudat.logic.managers;

import eu.eudat.data.query.items.item.funder.FunderCriteriaRequest;
import eu.eudat.logic.builders.model.models.FunderBuilder;
import eu.eudat.logic.proxy.config.exceptions.HugeResultSet;
import eu.eudat.logic.proxy.config.exceptions.NoURLFound;
import eu.eudat.logic.proxy.fetching.RemoteFetcher;
import eu.eudat.logic.services.ApiContext;
import eu.eudat.models.data.external.ExternalSourcesItemModel;
import eu.eudat.models.data.external.FundersExternalSourcesModel;
import eu.eudat.models.data.funder.Funder;
import eu.eudat.models.data.security.Principal;
import eu.eudat.queryable.QueryableList;
import org.springframework.stereotype.Component;

import java.util.Comparator;
import java.util.List;
import java.util.Map;

@Component
public class FunderManager {

	private ApiContext apiContext;
	private RemoteFetcher remoteFetcher;

	public FunderManager(ApiContext apiContext, RemoteFetcher remoteFetcher) {
		this.apiContext = apiContext;
		this.remoteFetcher = remoteFetcher;
	}

	public List<Funder> getCriteriaWithExternal(FunderCriteriaRequest funderCriteria, Principal principal) throws HugeResultSet, NoURLFound {
		eu.eudat.data.entities.UserInfo userInfo = new eu.eudat.data.entities.UserInfo();
		userInfo.setId(principal.getId());
		QueryableList<eu.eudat.data.entities.Funder> items = apiContext.getOperationsContext().getDatabaseRepository().getFunderDao().getWithCritetia(funderCriteria.getCriteria());
		//QueryableList<eu.eudat.data.entities.Funder> authItems = apiContext.getOperationsContext().getDatabaseRepository().getFunderDao().getAuthenticated(items, userInfo);
		List<Funder> funders = items.select(item -> new eu.eudat.models.data.funder.Funder().fromDataModel(item));
		List<Map<String, String>> remoteRepos = remoteFetcher.getFunders(funderCriteria.getCriteria().getLike());
		FundersExternalSourcesModel fundersExternalSourcesModel = new FundersExternalSourcesModel().fromExternalItem(remoteRepos);
		for (ExternalSourcesItemModel externalListingItem : fundersExternalSourcesModel) {
			eu.eudat.models.data.funder.Funder funder = apiContext.getOperationsContext().getBuilderFactory().getBuilder(FunderBuilder.class)
					.reference(externalListingItem.getRemoteId()).label(externalListingItem.getName())
					.status(eu.eudat.data.entities.Funder.Status.fromInteger(0))
					.build();

			funders.add(funder);
		}
		funders.sort(Comparator.comparing(x -> x.getLabel()));
		return funders;
	}
}
