package eu.eudat.logic.security.validators;

import eu.eudat.logic.security.customproviders.B2Access.B2AccessCustomProvider;
import eu.eudat.logic.security.customproviders.ORCID.ORCIDCustomProvider;
import eu.eudat.logic.security.validators.b2access.B2AccessTokenValidator;
import eu.eudat.logic.security.validators.facebook.FacebookTokenValidator;
import eu.eudat.logic.security.validators.google.GoogleTokenValidator;
import eu.eudat.logic.security.validators.linkedin.LinkedInTokenValidator;
import eu.eudat.logic.security.validators.orcid.ORCIDTokenValidator;
import eu.eudat.logic.security.validators.twitter.TwitterTokenValidator;
import eu.eudat.logic.services.ApiContext;
import eu.eudat.logic.services.operations.authentication.AuthenticationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;


@Service("tokenValidatorFactory")
public class TokenValidatorFactoryImpl implements TokenValidatorFactory {
    public enum LoginProvider {
        GOOGLE(1), FACEBOOK(2), TWITTER(3), LINKEDIN(4), NATIVELOGIN(5), B2_ACCESS(6), ORCID(7);

        private int value;

        private LoginProvider(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }

        public static LoginProvider fromInteger(int value) {
            switch (value) {
                case 1:
                    return GOOGLE;
                case 2:
                    return FACEBOOK;
                case 3:
                    return TWITTER;
                case 4:
                    return LINKEDIN;
                case 5:
                    return NATIVELOGIN;
                case 6:
                    return B2_ACCESS;
                case 7:
                    return ORCID;
                default:
                    throw new RuntimeException("Unsupported LoginProvider");
            }
        }
    }

    private ApiContext apiContext;
    private Environment environment;
    private AuthenticationService nonVerifiedUserAuthenticationService;
    private B2AccessCustomProvider b2AccessCustomProvider;
    private ORCIDCustomProvider orcidCustomProvider;

    @Autowired
    public TokenValidatorFactoryImpl(ApiContext apiContext, Environment environment, AuthenticationService nonVerifiedUserAuthenticationService, B2AccessCustomProvider b2AccessCustomProvider, ORCIDCustomProvider orcidCustomProvider) {
        this.apiContext = apiContext;
        this.environment = environment;
        this.nonVerifiedUserAuthenticationService = nonVerifiedUserAuthenticationService;
        this.b2AccessCustomProvider = b2AccessCustomProvider;
        this.orcidCustomProvider = orcidCustomProvider;
    }

    public TokenValidator getProvider(LoginProvider provider) {
        switch (provider) {
            case GOOGLE:
                return new GoogleTokenValidator(this.apiContext, this.environment, this.nonVerifiedUserAuthenticationService);
            case FACEBOOK:
                return new FacebookTokenValidator(this.apiContext, this.environment, this.nonVerifiedUserAuthenticationService);
            case LINKEDIN:
                return new LinkedInTokenValidator(this.apiContext, this.environment, this.nonVerifiedUserAuthenticationService);
            case TWITTER:
                return new TwitterTokenValidator(this.apiContext, this.environment, this.nonVerifiedUserAuthenticationService);
            case B2_ACCESS:
                return new B2AccessTokenValidator(this.environment, this.nonVerifiedUserAuthenticationService, this.b2AccessCustomProvider);
            case ORCID:
                return new ORCIDTokenValidator(this.environment, this.nonVerifiedUserAuthenticationService, this.orcidCustomProvider, this.apiContext);
            default:
                throw new RuntimeException("Login Provider Not Implemented");
        }
    }
}
