package eu.eudat.logic.security.validators.linkedin;

import eu.eudat.exceptions.security.UnauthorisedException;
import eu.eudat.logic.security.validators.TokenValidator;
import eu.eudat.logic.security.validators.TokenValidatorFactoryImpl;
import eu.eudat.logic.services.ApiContext;
import eu.eudat.logic.services.operations.authentication.AuthenticationService;
import eu.eudat.models.data.login.LoginInfo;
import eu.eudat.models.data.loginprovider.LoginProviderUser;
import eu.eudat.models.data.security.Principal;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.social.linkedin.api.LinkedIn;
import org.springframework.social.linkedin.api.LinkedInProfile;
import org.springframework.social.linkedin.connect.LinkedInServiceProvider;
import org.springframework.social.oauth2.AccessGrant;
import org.springframework.stereotype.Component;


@Component("linkedInTokenValidator")
public class LinkedInTokenValidator implements TokenValidator {

    private Environment environment;
    private ApiContext apiContext;
    private AuthenticationService nonVerifiedUserAuthenticationService;
    private LinkedInServiceProvider linkedInServiceProvider;

    @Autowired
    public LinkedInTokenValidator(ApiContext apiContext, Environment environment, AuthenticationService nonVerifiedUserAuthenticationService) {
        this.environment = environment;
        this.apiContext = apiContext;
        this.nonVerifiedUserAuthenticationService = nonVerifiedUserAuthenticationService;
        this.linkedInServiceProvider = new LinkedInServiceProvider(this.environment.getProperty("linkedin.login.clientId"), this.environment.getProperty("linkedin.login.clientSecret"));
    }

    @Override
    public Principal validateToken(LoginInfo credentials) {
        AccessGrant accessGrant = this.linkedInServiceProvider.getOAuthOperations().exchangeForAccess(credentials.getTicket(), this.environment.getProperty("linkedin.login.redirect_uri"), null);
        LinkedIn linkedInService = this.linkedInServiceProvider.getApi(accessGrant.getAccessToken());
        LinkedInProfile linkedInProfile = linkedInService.profileOperations().getUserProfile();
        LoginProviderUser user = new LoginProviderUser();

        if (linkedInProfile.getEmailAddress() == null)
            throw new UnauthorisedException("Cannot login user.LinkedIn account did not provide email");
        user.setEmail(linkedInProfile.getEmailAddress());
        user.setId(linkedInProfile.getId());
        user.setIsVerified(true); //TODO
        user.setAvatarUrl(linkedInProfile.getProfilePictureUrl());
        user.setName(linkedInProfile.getFirstName() + " " + linkedInProfile.getLastName());
        user.setProvider(TokenValidatorFactoryImpl.LoginProvider.LINKEDIN);
        user.setSecret(accessGrant.getAccessToken());
        return this.nonVerifiedUserAuthenticationService.Touch(user);
    }
}
