import { HttpClient, HttpHeaders, HttpResponse } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { environment } from '../../../../environments/environment';
import { BaseHttpParams } from '../../../common/http/base-http-params';
import { InterceptorType } from '../../../common/http/interceptors/interceptor-type';
import { DatasetProfileEditorModel } from '../../../ui/admin/dataset-profile/editor/dataset-profile-editor-model';
import { DatasetProfile } from '../../model/admin/dataset-profile/dataset-profile';
import { DataTableData } from '../../model/data-table/data-table-data';
import { DataTableRequest } from '../../model/data-table/data-table-request';
import { DatasetProfileDefinitionModel } from '../../model/dataset-profile-definition/dataset-profile-definition';
import { DatasetListingModel } from '../../model/dataset/dataset-listing';
import { DatasetProfileCriteria } from '../../query/dataset-profile/dataset-profile-criteria';
import { BaseHttpService } from '../http/base-http.service';

@Injectable()
export class DatasetProfileService {

	private actionUrl: string;
	private headers = new HttpHeaders();

	constructor(private http: BaseHttpService, private httpClient: HttpClient) {
		this.actionUrl = environment.Server + 'admin/';
	}

	createForm(data) {
		return this.http.post<DatasetProfileEditorModel>(this.actionUrl + 'addDmp', data);
	}

	updateForm(id, data) {
		return this.http.post<DatasetProfileEditorModel>(this.actionUrl + 'addDmp/' + id, data);
	}

	getDatasetProfileById(datasetProfileID) {
		return this.http.get<DatasetProfile>(this.actionUrl + 'get/' + datasetProfileID);
	}

	getPaged(dataTableRequest: DataTableRequest<DatasetProfileCriteria>): Observable<DataTableData<DatasetListingModel>> {
		return this.http.post<DataTableData<DatasetListingModel>>(this.actionUrl + 'datasetprofiles/getPaged', dataTableRequest);
	}

	preview(data: DatasetProfile): Observable<DatasetProfileDefinitionModel> {
		return this.http.post<DatasetProfileDefinitionModel>(this.actionUrl + 'preview', data);
	}

	clone(id: string): Observable<DatasetProfile> {
		return this.http.post<DatasetProfile>(this.actionUrl + 'datasetprofile/clone/' + id, {});
	}

	newVersion(id, data) {
		return this.http.post<DatasetProfileEditorModel>(this.actionUrl + 'newVersion/' + id, data);
	}

	delete(id: string, data): Observable<DatasetProfile> {
		//return this.http.post<DatasetProfile>(this.actionUrl + 'addDmp/' + id, data);
		return this.http.delete<DatasetProfile>(this.actionUrl + id, {});
	}

	downloadXML(id: string): Observable<HttpResponse<Blob>> {
		let headerXml: HttpHeaders = this.headers.set('Content-Type', 'application/xml')
		return this.httpClient.get(this.actionUrl + 'getXml/' + id, { responseType: 'blob', observe: 'response', headers: headerXml });
	}

	uploadFile(file: FileList, labelSent: string): Observable<DataTableData<DatasetListingModel>> {
		const params = new BaseHttpParams();
		params.interceptorContext = {
			excludedInterceptors: [InterceptorType.JSONContentType]
		};
		const formData = new FormData();
		formData.append('file', file[0], labelSent);
		return this.http.post(this.actionUrl + "upload", formData, { params: params });
	}
}
