import { Status } from "../../../../core/common/enum/Status";
import { ProjectModel } from "../../../../core/model/project/project";
import { ValidationErrorModel } from "../../../../common/forms/validation/error-model/validation-error-model";
import { ValidationContext } from "../../../../common/forms/validation/validation-context";
import { FormGroup, FormBuilder, Validators } from "@angular/forms";
import { BackendErrorValidator } from "../../../../common/forms/validation/custom-validator";
import { ValidJsonValidator } from "../../../../library/auto-complete/auto-complete-custom-validator";

export class ProjectFormModel {
	public id: string;
	public label?: string;
	public status: Status = Status.Active;
	public description: String;
	public existProject: ProjectModel;
	public validationErrorModel: ValidationErrorModel = new ValidationErrorModel();


	fromModel(item: ProjectModel): ProjectFormModel {
		this.existProject = item;
		return this;
	}

	buildForm(context: ValidationContext = null, disabled: boolean = false): FormGroup {
		if (context == null) { context = this.createValidationContext(); }

		const formGroup = new FormBuilder().group({
			id: [{ value: this.id, disabled: disabled }, context.getValidation('id').validators],
			label: [{ value: this.label, disabled: disabled }, context.getValidation('label').validators],
			status: [{ value: this.status, disabled: disabled }, context.getValidation('status').validators],
			description: [{ value: this.description, disabled: disabled }, context.getValidation('description').validators],
			existProject: [{ value: this.existProject, disabled: disabled }, context.getValidation('existProject').validators],
		});
		return formGroup;
	}

	createValidationContext(): ValidationContext {
		const baseContext: ValidationContext = new ValidationContext();
		baseContext.validation.push({ key: 'id', validators: [] });
		baseContext.validation.push({ key: 'label', validators: [Validators.required, BackendErrorValidator(this.validationErrorModel, 'label')] });
		baseContext.validation.push({ key: 'status', validators: [] });
		baseContext.validation.push({ key: 'description', validators: [Validators.required, BackendErrorValidator(this.validationErrorModel, 'description')] });
		baseContext.validation.push({ key: 'existProject', validators: [Validators.required, ValidJsonValidator, BackendErrorValidator(this.validationErrorModel, 'existProject')] });
		return baseContext;
	}

}
